#include "PhysicsAPI.h"
#include "Object.h"
#include "DynamicObject.h"
#include "Player.h"
#include "Level.h"
#include "AttatchmentMassDriver.h"
#include "Game.h"
#include "CollisionManager.h"

using namespace Oyster;

using namespace GameLogic;

	void PlayerVBox(Player &player, DynamicObject &box, Oyster::Math::Float kineticEnergyLoss);
	void PlayerVObject(Player &player, Object &obj, Oyster::Math::Float kineticEnergyLoss);

	//Physics::ICustomBody::SubscriptMessage
	void Player::PlayerCollision(Oyster::Physics::ICustomBody *rigidBodyPlayer, Oyster::Physics::ICustomBody *obj, Oyster::Math::Float kineticEnergyLoss)
	{
		
		Player *player = ((Game::PlayerData*)(rigidBodyPlayer->GetCustomTag()))->player;
		Object *realObj = (Object*)obj->GetCustomTag(); //needs to be changed?

		switch (realObj->GetObjectType())
		{
		case OBJECT_TYPE::OBJECT_TYPE_GENERIC:
			PlayerVObject(*player,*realObj, kineticEnergyLoss);
			//return Physics::ICustomBody::SubscriptMessage_none;
			break;
		
		case OBJECT_TYPE::OBJECT_TYPE_BOX:
			PlayerVBox(*player,(*(DynamicObject*) realObj), kineticEnergyLoss);
			//return Physics::ICustomBody::SubscriptMessage_none;
			break;
		case OBJECT_TYPE::OBJECT_TYPE_PLAYER:
			//return Physics::ICustomBody::SubscriptMessage_none;
			break;
		case OBJECT_TYPE::OBJECT_TYPE_WORLD:
			int test = 5;
			break;
		}

		//return Physics::ICustomBody::SubscriptMessage_none;
	}
		
	void PlayerVBox(Player &player, DynamicObject &box, Oyster::Math::Float kineticEnergyLoss)
	{
		//use kinetic energyloss of the collision in order too determin how much damage to take
		//use as part of the damage algorithm
		player.DamageLife(20);
	}

	void PlayerVObject(Player &player, Object &obj, Oyster::Math::Float kineticEnergyLoss)
	{
		//Collision between a player and a general static or dynamic object
		//use kinetic energyloss of the collision in order too determin how much damage to take
		//use as part of the damage algorithm
		int damageDone = 0;
		int forceThreashHold = 200;

		if(kineticEnergyLoss > forceThreashHold) //should only take damage if the force is high enough
		{
			damageDone = kineticEnergyLoss * 0.10f;
			player.DamageLife(damageDone);
		}
		
	}	
	Oyster::Physics::ICustomBody::SubscriptMessage Object::DefaultCollisionBefore(Oyster::Physics::ICustomBody *rigidBodyLevel, Oyster::Physics::ICustomBody *obj)
	{
		return Physics::ICustomBody::SubscriptMessage_none;
	}
	Oyster::Physics::ICustomBody::SubscriptMessage Object::DefaultCollisionAfter(Oyster::Physics::ICustomBody *rigidBodyLevel, Oyster::Physics::ICustomBody *obj, Oyster::Math::Float kineticEnergyLoss)
	{
		return Physics::ICustomBody::SubscriptMessage_none;
	}
	//Oyster::Physics::ICustomBody::SubscriptMessage
	Oyster::Physics::ICustomBody::SubscriptMessage Level::LevelCollisionBefore(Oyster::Physics::ICustomBody *rigidBodyLevel, Oyster::Physics::ICustomBody *obj)
	{
		return Physics::ICustomBody::SubscriptMessage_ignore_collision_response;
	}

	Oyster::Physics::ICustomBody::SubscriptMessage CollisionManager::IgnoreCollision(Oyster::Physics::ICustomBody *rigidBody, Oyster::Physics::ICustomBody *obj)
	{
		return Physics::ICustomBody::SubscriptMessage_ignore_collision_response;
	}


	Oyster::Physics::ICustomBody::SubscriptMessage Level::LevelCollisionAfter(Oyster::Physics::ICustomBody *rigidBodyLevel, Oyster::Physics::ICustomBody *obj, Oyster::Math::Float kineticEnergyLoss)
	{
		return Physics::ICustomBody::SubscriptMessage_ignore_collision_response;
	}

	void AttatchmentMassDriver::ForcePushAction(Oyster::Physics::ICustomBody *obj, void *args)
	{
		Oyster::Physics::ICustomBody::State state;
		Object *realObj = (Object*)obj->GetCustomTag();

		if(realObj->GetObjectType() == OBJECT_TYPE_PLAYER || realObj->GetObjectType() == OBJECT_TYPE_WORLD)
			return;

		state = obj->GetState();
		state.ApplyLinearImpulse(((forcePushData*)(args))->pushForce);
		obj->SetState(state);
	}

	void AttatchmentMassDriver::AttemptPickUp(Oyster::Physics::ICustomBody *obj, void* args)
	{
		AttatchmentMassDriver *weapon = ((AttatchmentMassDriver*)args);

		if(weapon->hasObject)
		{
			//do nothing
		}
		else
		{
			Object* realObj = (Object*)(obj->GetCustomTag());
			//check so that it is an object that you can pickup

			switch(realObj->GetObjectType())
			{
			case OBJECT_TYPE::OBJECT_TYPE_BOX:
				obj->SetGravity(true); //will now ignore gravity, dont mind the naming
				//move obj to limbo in physics to make sure it wont collide with anything
				weapon->heldObject = obj; //weapon now holds the object
				weapon->hasObject = true;

				break;
			}
			
		}
		

	}